import jas.engine.*;
import jas.events.*;
import jas.io.*;

import java.io.*;
import java.util.*;

/**
 * The Model contains the Units
 *
 * @author Pietro Terna, with contributions of Marco Remondino, Dario Landini,
 * Nicola Ormezzano and Paolo Pelligra
 * @author Michele Sonnessa: JAS porting 
 */
public class ESFrameModel extends SimModel
{
	ParameterFrame parameterFrame;

	// simulation parameters
	public static boolean verbose = true;

	/** the total number of operating Units we are using*/
	public int totalUnitNumber;
	/* the max number of steps to be done to complete an order */
	public int maxStepNumber;
	/** the max length of a step in a recipe (in units of time) */
	public int maxStepLength;
	/** the total production time used by done orders */
	public float totalProductionTime;
	/** the total length of the recipes of the done orders */
	public float totalRecipeLength;
	
	/** the order generator */
	public OrderGenerator orderGenerator;
	
	/** the input class for unit parameters*/
	public UnitParameters unitParameters;

	/** the list of the operating units */
	public ArrayList unitList;
	/** its iterator */
	public ListIterator unitListIndex;
	
	/** the tool to assign order to units */
	AssigningTool assigningTool;
	
	public void setParameters()
	{
		// Now fill in various simulation parameters with default values.
		totalUnitNumber 	= 10;
		maxStepNumber			= 30;
		maxStepLength			= 1;

		parameterFrame = new ParameterFrame(this, "Model",
								new ParametersBag(path + "ModelParamForm.pForm.xml"));

		addSimWindow(parameterFrame);
	}

	/** Build the model objects, then
	 * the model schedule is built, the data structures
	 * that define the simulation of time in the model. The core is an
	 * actionGroup that has a list of actions. That's then put in a
	 * Schedule. 
	 */
	public void buildModel()
	{
		buildObjects();
		buildActions();
	}

	private void buildObjects ()
	{
		// create the class to read acounting parameters
    unitParameters = new UnitParameters(totalUnitNumber);
		unitParameters.readUnitData();
		
		// the list of the production units
		unitList = new ArrayList ();
		// its iterator
		unitListIndex = unitList.listIterator();

		// the instance of assigningTool
		assigningTool = new AssigningTool ( unitList);

		// the orderGenerator
		orderGenerator = new OrderGenerator
		    (maxStepNumber, maxStepLength, unitList,
		     this, assigningTool);

		// create the units
		for (int i = 1; i <= totalUnitNumber; i++)
		{
			Unit aUnit = new Unit (this, assigningTool);
			unitParameters.setParametersTo(i, aUnit);
		
			// add the unit to the list
			unitList.add(aUnit);
		}

		// creating the dictionary in orderGenerator
		orderGenerator.setDictionary();

	}

	public void buildActions ()
	{

		SimGroupEvent modelActions2 = eventList.scheduleGroup(0, 1);
		SimGroupEvent modelActions2generator = eventList.scheduleGroup(0, 1);
		SimGroupEvent modelActions2b = eventList.scheduleGroup(0, 1);

		/* producing */
		modelActions2.addCollectionEvent(unitList, getObjectClass("Unit"), "unitStep1");

		/* a new order; this step is placed here, after the production step,
		 * to align the diffusion of the order forms (orders under execution
		 * - next step - or new ones - this step -) */
		modelActions2generator.addEvent(orderGenerator, "createRandomOrderWithNSteps");
		modelActions2b.addCollectionEvent(unitList, getObjectClass("Unit"), "unitStep2");

	}

	
	/** the method returns the list of the units */
	public List getUnitList(){
		return unitList;
	}

	/** record total production time and total recipe length */
	public void setProductionTimeAndRecipeLength(float pt, float rl) 
	{
		totalProductionTime += pt;
		totalRecipeLength   += rl;
		if(verbose)
		    System.out.println("totalProductionTime " + totalProductionTime +
				       " totalRecipeLength " + totalRecipeLength +
				       " ratio " + ((float)
						    totalProductionTime)/totalRecipeLength);
	}

	/** report the ratio totalProductionTime/totalRecipeLength */
	public double getTimeLengthRatio()
	{
		if(totalRecipeLength != 0)
		    return totalProductionTime / totalRecipeLength;
		else 
				return  1.0; // 1 is a good choice, being 0 totalRecipeLength
		                 // in the first simulation days and being 1
		                 // the actual ratio of the first
		                 // concluded orders
	}

}


