// Unit.java

import java.io.*;
import java.util.*;
import jas.engine.Sim;

/**
 * The Unit class instances are micro unit in our virtual enterprise;
 * i.e. the units where the steps required to build a product (to fulfill
 * an order) are done
 *
 * @author Pietro Terna (with Marco Remondino, Dario Landini, Nicola
 * Ormezzano and Paolo Pelligra contributions)
 */
public class Unit 
{
	
	/** the number identifying the unit*/
	public int unitNumber;
	/** the unit production phase */
	public int unitProductionPhase;
	
	/** the list of the orders to be executed */
	public LinkedList  waitingList;
	/** the list containing the orders (if any) performed in a day */
	public LinkedList dailyProductionList;
	/** the modelSwarm (to which we apply the
	 * setProductionTimeAndRecipeLength(...) method */
	ESFrameModel eSFrameModel;
	/** the temporary address of an existing unit */
	public Unit aUnit;
	/** the first order to be executed */
	public Order firstOrder;
	
	/** the instance of AssigningTool we use */
	AssigningTool assigningTool;
	
	/**
	 * the constructor for Unit
	 */
	public Unit (ESFrameModel mo,
		 AssigningTool at) {
	
	
	// the production waiting list of the unit
	waitingList = new LinkedList ();
	// the model
	eSFrameModel = mo;
	// the daily list of finished orders
	dailyProductionList= new LinkedList();
	// the assigning tool
		assigningTool=at;
	
	   }
	
	    /** set unit number */
	public void setUnitNumber(int un)
	{
	// the unit id number.
	unitNumber=un;
	// announce the unit to the console
	if(ESFrameModel.verbose)
	System.out.println("Unit number " + unitNumber + " has been created.");
	 }
	
	/** get unit number */
	public int getUnitNumber()
	{
	// the unit id number.
	return unitNumber;
	}
	
	
	/**
	 * production:<br><br>
	 *
	 * at the beginning of each time tick in the 'day', each unit looks for
	 * the order(s) (if any) to be executed; the production
	 * cycle is 'one unit of time'
	 * in a step of the recipe of an order in a tick of a day;<br><br>
	 */
	public void unitStep1 ()
	{
	int i, ii;
	
	if(ESFrameModel.verbose)
	    System.out.println("Step 1 in unit " + unitNumber +
			       "; waitingList count " +
			       waitingList.size());
	
	// the production
	
	// after the specific step in recipe production we place the order(s)
	// in a daily production
	// list, which will be used at unitStep2 to diffuse production
	// results
	
	
	if(waitingList.size() > 0){
	    firstOrder=(Order)waitingList.removeFirst();
	
	    if(firstOrder.getNextStep()!=getProductionPhase()){
		System.out.println("Unrecoverable error (case2) in unit "
				   + unitNumber + " (order number "
				   + firstOrder.getOrderNumber()+ " step "
				   + firstOrder.getNextStep()+")");
		System.exit(1);}
	
	
	    firstOrder.setDoneStep(unitNumber);
	    dailyProductionList.addLast(firstOrder);
	}
	
	} // concluding step1
	
	/**
	 * diffusion and accounting:<br><br>
	 *
	 * sending the orders to the subsequent production units;<br><br>
	 *
	 * making accounting operations
	 */
	public void unitStep2 () {
	float timeSpent;
	
	if(ESFrameModel.verbose)
	    System.out.println("Step 2 in unit " + unitNumber +
			       " dailyProductionList count " +
			       dailyProductionList.size());
	
	while(dailyProductionList.size()>0)
	    {
		firstOrder=(Order)dailyProductionList.removeFirst();
	
		// do we have concluded the production step?
		// if yes ... we drop the object
		if (firstOrder.getNextStep()==0)
		    {
			if(ESFrameModel.verbose)
			    System.out.println("drop order # " +
					       firstOrder.getOrderNumber() +
					       " startingTimeUnit " +
					       firstOrder.getStartingTimeUnit() +
					       " endTimeUnit " +
					       Sim.getAbsoluteTime() +
					       " stepNumber " +
					       firstOrder.getStepNumber() +
					       " ratio " +
					       ( (float)
						(Sim.getAbsoluteTime() -
						firstOrder.getStartingTimeUnit()))/
					        firstOrder.getStepNumber());
			// the production is concluded, so we inform
			// our environment (the model) of the time spent
			// and of the # of steps in recipe
			timeSpent=Sim.getAbsoluteTime() -
			    firstOrder.getStartingTimeUnit();
			eSFrameModel.setProductionTimeAndRecipeLength(
			  timeSpent,
			  firstOrder.getNumberOfDirectStepsDone());
			if(ESFrameModel.verbose)
			    System.out.println("Order #" +
					       firstOrder.orderNumber +
					       " has been completed; time spent in ticks: " +
					       timeSpent +
					       " expected time in ticks " +
					       firstOrder.getStepNumber() +
					       " or with stand alone batches " +
					       firstOrder.getNumberOfDirectStepsDone());
	          // MM firstOrder.drop();
		    }
	
		// if not ... we send it to the subsequent production unit
		else assigningTool.assign(firstOrder);
	    }
	
	}
	
	/** set unit production phases */
	public void setProductionPhase(int ph)
	{
	unitProductionPhase = ph;
	}
	
	
	/**
	 * return the production phases of the unit
	 */
	public int getProductionPhase()
	{
	return unitProductionPhase;
	}
	
	/**
	 * Putting an order in the production waitingList of the unit
	 * (using a related object if any)
	 */
	public void setWaitingList(Order anOrder)
	{
	int i;
	
	// normal case:
	waitingList.addLast(anOrder);
	
	if(ESFrameModel.verbose)
	    System.out.println("The unit " + unitNumber +
			       " has received the order # "
				       + anOrder.getOrderNumber());
	    }

}
